@@ -39,6 +39,7 @@
 #include <mailutils/stream.h>
 #include <mailutils/url.h>
 #include <mailutils/nls.h>
+#include <mailutils/tls.h>
 
 static char* show_field;
 static int show_to;
@@ -161,6 +162,9 @@ static const char *frm_argp_capa[] = {
   "common",
   "license",
   "mailbox",
+#ifdef WITH_TLS
+  "tls",
+#endif
   NULL
 };
 
@@ -174,7 +178,7 @@ get_personal (header_t hdr, const char *field, char *personal, size_t buflen)
   /* Empty string.  */
   *hfield = '\0';
 
-  status = header_get_value (hdr, field, hfield, sizeof (hfield), NULL);
+  status = header_get_value_unfold (hdr, field, hfield, sizeof (hfield), NULL);
   if (status == 0)
     {
       address_t address = NULL;
@@ -242,8 +246,8 @@ action (observer_t o, size_t type)
 	if (show_field)
 	  {
 	    char hfield[256];
-	    int status = header_get_value (hdr, show_field, hfield,
-					   sizeof (hfield), NULL);
+	    int status = header_get_value_unfold (hdr, show_field, hfield,
+						  sizeof (hfield), NULL);
 	    if (status == 0)
 	      printf ("%s", hfield);
 	  }
@@ -272,8 +276,9 @@ action (observer_t o, size_t type)
 	if (show_subject)
 	  {
 	    char hsubject[64];
-	    int status = header_get_value (hdr, MU_HEADER_SUBJECT, hsubject,
-					   sizeof (hsubject), NULL);
+	    int status = header_get_value_unfold (hdr, MU_HEADER_SUBJECT,
+						  hsubject,
+						  sizeof (hsubject), NULL);
 	    if(status == 0)
 	      printf("%s", hsubject);
 	  }
@@ -307,6 +312,9 @@ main(int argc, char **argv)
   mu_init_nls ();
 
   mu_argp_init (program_version, NULL);
+#ifdef WITH_TLS
+  mu_tls_init_client_argp ();
+#endif
   mu_argp_parse (&argp, &argc, &argv, 0, frm_argp_capa, &c, NULL);
 
   /* have an argument */
@@ -322,6 +330,7 @@ main(int argc, char **argv)
     registrar_get_list (&bookie);
     list_append (bookie, mbox_record);
     list_append (bookie, path_record);
+    list_append (bookie, mh_record);
     list_append (bookie, pop_record);
     list_append (bookie, imap_record);
   }
