@@ -28,6 +28,7 @@ extern int list_create   __P ((list_t *));
 extern void list_destroy __P ((list_t *));
 extern int list_append   __P ((list_t, void *item));
 extern int list_prepend  __P ((list_t, void *item));
+extern int list_insert   __P ((list_t list, void *item, void *new_item));
 extern int list_is_empty __P ((list_t));
 extern int list_count    __P ((list_t, size_t *pcount));
 extern int list_remove   __P ((list_t, void *item));
@@ -41,6 +42,8 @@ extern int list_do       __P ((list_t list, list_action_t * action, void *cbdata
 typedef int (*list_comparator_t) __PMT((const void*, const void*));
 
 extern list_comparator_t list_set_comparator __P((list_t, list_comparator_t));
+
+extern int list_set_destroy_item __P ((list_t list, void (*destoy_item) __P((void *))));
   
 #ifdef __cplusplus
 }
