@@ -39,7 +39,7 @@
 # define _(String) (String)
 # define N_(String) String
 # define dgettext(domain, msgid) (msgid)
-# define ngettext(sg,pl,cnt) (pl)
+# define ngettext(sg,pl,cnt) (cnt == 1) ? (sg) : (pl)
 # define textdomain(Domain)
 # define bindtextdomain(Package, Directory)
 #endif /* ENABLE_NLS */
