@@ -48,6 +48,8 @@
 #include <mailutils/header.h>
 #include <mailutils/observer.h>
 #include <mailutils/stream.h>
+#include <mailutils/argcv.h>
+#include <mailutils/tls.h>
 
 /* For dbg purposes set to one to see different level of traffic.  */
 /* Print to stderr the command sent to the IMAP server.  */
@@ -96,9 +98,6 @@ static int  folder_imap_subscribe   __P ((folder_t, const char *));
 static int  folder_imap_unsubscribe __P ((folder_t, const char *));
 static int  folder_imap_get_authority __P ((folder_t, authority_t *));
 
-static int  authenticate_imap_login __P ((authority_t));
-static int  authenticate_imap_sasl_anon __P ((authority_t));
-
 /* FETCH  */
 static int  imap_fetch              __P ((f_imap_t));
 static int  imap_rfc822             __P ((f_imap_t, char **));
@@ -125,92 +124,15 @@ static int  imap_mailbox_name_match __P ((const char* pattern, const char* mailb
 
 static int  imap_token              __P ((char *, size_t, char **));
 
-/* Initialize the concrete IMAP mailbox: overload the folder functions  */
-int
-_folder_imap_init (folder_t folder)
-{
-  int status;
-  f_imap_t f_imap;
+/* Capability */
+static int  parse_capa              __P ((f_imap_t f_imap, char *str));
+static int  read_capa               __P ((f_imap_t f_imap, int force));
+static int  check_capa              __P ((f_imap_t f_imap, char *capa));
 
-  /* Set the authority early:
-     (1) so we can check for errors.
-     (2) allow the client to get the authority for setting the ticket
-     before the open.  */
-  status = folder_imap_get_authority (folder, NULL);
-  if (status != 0)
-    return status;
+
+/* Authentication methods */
 
-  f_imap = folder->data = calloc (1, sizeof (*f_imap));
-  if (f_imap == NULL)
-    return ENOMEM;
-
-  f_imap->folder = folder;
-  f_imap->state = IMAP_NO_STATE;
-
-  folder->_destroy = folder_imap_destroy;
-
-  folder->_open = folder_imap_open;
-  folder->_close = folder_imap_close;
-
-  folder->_list = folder_imap_list;
-  folder->_lsub = folder_imap_lsub;
-  folder->_subscribe = folder_imap_subscribe;
-  folder->_unsubscribe = folder_imap_unsubscribe;
-  folder->_delete = folder_imap_delete;
-  folder->_rename = folder_imap_rename;
-
-  return 0;
-}
-
-/* Destroy the folder resources.  */
-static void
-folder_imap_destroy (folder_t folder)
-{
-  if (folder->data)
-    {
-      f_imap_t f_imap = folder->data;
-      if (f_imap->buffer)
-	free (f_imap->buffer);
-      if (f_imap->capa)
-	free (f_imap->capa);
-      free (f_imap);
-      folder->data = NULL;
-    }
-}
-
-static int
-folder_imap_get_authority (folder_t folder, authority_t *pauth)
-{
-  int status = 0;
-  if (folder->authority == NULL)
-    {
-      /* assert (folder->url); */
-      if (folder->url == NULL)
-	return EINVAL;
-
-      if (folder->url->auth == NULL
-	  || strcasecmp (folder->url->auth, "*") == 0)
-	{
-	  status = authority_create (&folder->authority, NULL, folder);
-	  authority_set_authenticate (folder->authority,
-				      authenticate_imap_login, folder);
-	}
-      else if (strcasecmp (folder->url->auth, "anon") == 0)
-	{
-	  status = authority_create (&folder->authority, NULL, folder);
-	  authority_set_authenticate (folder->authority,
-				      authenticate_imap_sasl_anon, folder);
-	}
-      else
-	{
-	  /* Not a supported authentication mechanism. */
-	  status = ENOSYS;
-	}
-    }
-  if (pauth)
-    *pauth = folder->authority;
-  return status;
-}
+typedef int (*auth_method_t) __P((authority_t));
 
 /* Simple User/pass authentication for imap.  */
 static int
@@ -221,6 +143,12 @@ authenticate_imap_login (authority_t auth)
   ticket_t ticket;
   int status = 0;
 
+  if (check_capa (f_imap, "LOGINDISABLED") == 0)
+    {
+      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, "LOGIN command disabled\n");
+      return ENOSYS;
+    }
+  
   switch (f_imap->state)
     {
     case IMAP_AUTH:
@@ -251,15 +179,17 @@ authenticate_imap_login (authority_t auth)
 	    url_get_passwd (folder->url, f_imap->passwd, n + 1, NULL);
 	  }
 
-	if (f_imap->user == NULL || f_imap->passwd == NULL)
-	  {
-	    CHECK_ERROR_CLOSE (folder, f_imap, EINVAL);
-	  }
-	status = imap_writeline (f_imap, "g%u LOGIN %s %s\r\n",
+	if (f_imap->user == NULL)
+	  return MU_ERR_NOUSERNAME;
+	  
+	if (f_imap->passwd == NULL)
+	  return MU_ERR_NOPASSWORD;
+	  
+	status = imap_writeline (f_imap, "g%u LOGIN \"%s\" \"%s\"\r\n",
 				 f_imap->seq, f_imap->user, f_imap->passwd);
 	CHECK_ERROR_CLOSE(folder, f_imap, status);
 	FOLDER_DEBUG2 (folder, MU_DEBUG_PROT, "g%u LOGIN %s *\n",
-	    f_imap->seq, f_imap->user);
+		       f_imap->seq, f_imap->user);
 	f_imap->seq++;
 	free (f_imap->user);
 	f_imap->user = NULL;
@@ -281,7 +211,8 @@ authenticate_imap_login (authority_t auth)
     case IMAP_LOGIN_ACK:
       /* Get the login ack.  */
       status = imap_parse (f_imap);
-      CHECK_EAGAIN (f_imap, status);
+      if (status)
+	return status;
       FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_AUTH_DONE;
 
@@ -344,6 +275,13 @@ authenticate_imap_sasl_anon (authority_t auth)
 
   assert (f_imap->state == IMAP_AUTH);
 
+  if (check_capa (f_imap, "AUTH=ANONYMOUS"))
+    {
+      FOLDER_DEBUG0 (folder, MU_DEBUG_PROT,
+		     "ANONYMOUS capability not present\n");
+      return ENOSYS;
+    }
+
   switch (f_imap->auth_state)
     {
     case IMAP_AUTH_ANON_REQ_WRITE:
@@ -400,6 +338,238 @@ authenticate_imap_sasl_anon (authority_t auth)
   return 0;
 }
 
+struct auth_tab
+{
+  char *name;
+  auth_method_t method;
+};
+
+/* NOTE: The ordering of methods in this table is from most secure
+   to less secure. */
+
+static struct auth_tab auth_tab[] = {
+  { "login", authenticate_imap_login },
+  { "anon", authenticate_imap_sasl_anon },
+  { NULL }
+};
+
+static auth_method_t
+find_auth_method (const char *name)
+{
+  struct auth_tab *p;
+
+  for (p = auth_tab; p->name; p++)
+    if (strcasecmp (p->name, name) == 0)
+      return p->method;
+
+  return NULL;
+}
+
+static int
+authenticate_imap_select (authority_t auth)
+{
+  folder_t folder = authority_get_owner (auth);
+  f_imap_t f_imap = folder->data;
+  struct auth_tab *p;
+  int status = MU_ERR_AUTH_FAILURE;
+  
+  for (p = auth_tab; status && p->name; p++)
+    {
+      f_imap->state = IMAP_AUTH;
+      status = p->method (auth);
+    }
+
+  return status;
+}
+
+
+
+
+/* Initialize the concrete IMAP mailbox: overload the folder functions  */
+int
+_folder_imap_init (folder_t folder)
+{
+  int status;
+  f_imap_t f_imap;
+
+  /* Set the authority early:
+     (1) so we can check for errors.
+     (2) allow the client to get the authority for setting the ticket
+     before the open.  */
+  status = folder_imap_get_authority (folder, NULL);
+  if (status != 0)
+    return status;
+
+  f_imap = folder->data = calloc (1, sizeof (*f_imap));
+  if (f_imap == NULL)
+    return ENOMEM;
+
+  f_imap->folder = folder;
+  f_imap->state = IMAP_NO_STATE;
+
+  folder->_destroy = folder_imap_destroy;
+
+  folder->_open = folder_imap_open;
+  folder->_close = folder_imap_close;
+
+  folder->_list = folder_imap_list;
+  folder->_lsub = folder_imap_lsub;
+  folder->_subscribe = folder_imap_subscribe;
+  folder->_unsubscribe = folder_imap_unsubscribe;
+  folder->_delete = folder_imap_delete;
+  folder->_rename = folder_imap_rename;
+
+  return 0;
+}
+
+/* Destroy the folder resources.  */
+static void
+folder_imap_destroy (folder_t folder)
+{
+  if (folder->data)
+    {
+      f_imap_t f_imap = folder->data;
+      if (f_imap->buffer)
+	free (f_imap->buffer);
+      if (f_imap->capav)
+	argcv_free (f_imap->capac, f_imap->capav);
+      free (f_imap);
+      folder->data = NULL;
+    }
+}
+
+static int
+folder_set_auth_method (folder_t folder, auth_method_t method)
+{
+  if (!folder->authority)
+    {
+      int status = authority_create (&folder->authority, NULL, folder);
+      if (status)
+	return status;
+    }
+  return authority_set_authenticate (folder->authority, method, folder);
+}
+
+static int
+folder_imap_get_authority (folder_t folder, authority_t *pauth)
+{
+  int status = 0;
+  if (folder->authority == NULL)
+    {
+      /* assert (folder->url); */
+      if (folder->url == NULL)
+	return EINVAL;
+
+      if (folder->url->auth == NULL
+	  || strcasecmp (folder->url->auth, "*") == 0)
+	{
+	  status = folder_set_auth_method (folder, authenticate_imap_select);
+	}
+      else 
+	{
+	  char *p, *sp;
+
+	  for (p = strtok_r (folder->url->auth, ",", &sp);
+	       status == 0 && p;
+	       p = strtok_r (NULL, ",", &sp))
+	    {
+	      auth_method_t method = find_auth_method (p);
+	      if (method)
+		status = folder_set_auth_method (folder, method);
+	      else
+		status = MU_ERR_BAD_AUTH_SCHEME;
+	    }		  
+	}
+    }
+
+  if (status)
+    return status;
+  
+  if (pauth)
+    *pauth = folder->authority;
+  return status;
+}
+
+
+/* Capability handling */
+static int
+parse_capa (f_imap_t f_imap, char *str)
+{
+  if (f_imap->capav)
+    argcv_free (f_imap->capac, f_imap->capav);
+  return argcv_get (str, "", NULL, &f_imap->capac, &f_imap->capav);
+}
+
+static int
+read_capa (f_imap_t f_imap, int force)
+{
+  int status = 0;
+  
+  if (force)
+    {
+      argcv_free (f_imap->capac, f_imap->capav);
+      f_imap->capac = 0;
+      f_imap->capav = NULL;
+    }
+  
+  if (!f_imap->capav)
+    {
+      status = imap_writeline (f_imap, "g%u CAPABILITY\r\n",
+			       f_imap->seq++);
+      status = imap_send (f_imap);
+      status = imap_parse (f_imap);
+    }
+  return status;
+}
+
+static int
+check_capa (f_imap_t f_imap, char *capa)
+{
+  int i;
+
+  read_capa (f_imap, 0);
+  for (i = 0; i < f_imap->capac; i++)
+    if (strcasecmp (f_imap->capav[i], capa) == 0)
+      return 0;
+  return 1;
+}
+
+
+static int
+tls (folder_t folder)
+{
+#ifdef WITH_TLS
+  int status;
+  f_imap_t f_imap = folder->data;
+
+  if (!mu_tls_enable || check_capa (f_imap, "STARTTLS"))
+    return -1;
+  
+  FOLDER_DEBUG1 (folder, MU_DEBUG_PROT, "g%u STARTTLS\n", f_imap->seq);
+  status = imap_writeline (f_imap, "g%u STARTTLS\r\n",
+			   f_imap->seq++, f_imap->user, f_imap->passwd);
+  CHECK_ERROR (f_imap, status);
+  status = imap_send (f_imap);
+  CHECK_ERROR (f_imap, status);
+  status = imap_parse (f_imap);
+  if (status == 0)
+    {
+      stream_t str;
+      status = tls_stream_create_client_from_tcp (&str, folder->stream, 0);
+      CHECK_ERROR (f_imap, status);
+      status = stream_open (str);
+      if (status == 0)
+	folder->stream = str;
+      FOLDER_DEBUG1 (folder, MU_DEBUG_PROT, "TLS negotiation %s\n",
+		     status == 0 ? "succeeded" : "failed");
+      read_capa (f_imap, 1);
+    }
+  return status;
+#else
+  return -1;
+#endif
+}
+
 /* Create/Open the stream for IMAP.  */
 static int
 folder_imap_open (folder_t folder, int flags)
@@ -486,8 +656,8 @@ folder_imap_open (folder_t folder, int flags)
         CHECK_EAGAIN (f_imap, status);
 	f_imap->ptr = f_imap->buffer;
         FOLDER_DEBUG0 (folder, MU_DEBUG_PROT, f_imap->buffer);
-	/* Are they open for business ?  The server send an untag response
-	   for greeting. Thenically it can be OK/PREAUTH/BYE.  The BYE is
+	/* Are they open for business ?  The server send an untagged response
+	   for greeting. Tecnically it can be OK/PREAUTH/BYE.  The BYE is
 	   the one that we do not want, server being unfriendly.  */
 	if (strncasecmp (f_imap->buffer, "* PREAUTH", 9) == 0)
 	  {
@@ -500,14 +670,24 @@ folder_imap_open (folder_t folder, int flags)
             f_imap->state = IMAP_AUTH;
 	  }
       }
-
+      tls(folder);
+      
     case IMAP_AUTH:
     case IMAP_LOGIN:
     case IMAP_LOGIN_ACK:
       assert (folder->authority);
       {
 	status = authority_authenticate (folder->authority);
-	CHECK_EAGAIN (f_imap, status);
+	if (status)
+	  {
+	    /* Fake folder_imap_close into closing the folder.
+	       FIXME: The entire state machine should probably
+	       be revised... */
+	    f_imap->isopen++;
+	    f_imap->state = IMAP_NO_STATE;
+	    folder_imap_close (folder);
+	    return status;
+	  }
       }
 
     case IMAP_AUTH_DONE:
@@ -979,7 +1159,7 @@ imap_literal_string (f_imap_t f_imap, char **ptr)
     }
 
   /* The (len + 1) in the for is to count the strip '\r' by imap_readline.  */
-  for (len0 = len = total = 0; total < f_imap->string.nleft; total += (len + 1))
+  for (len0 = len = total = 0; total < f_imap->string.nleft; total += len + 1)
     {
       status = imap_readline (f_imap);
       if (DEBUG_SHOW_DATA)
@@ -1948,7 +2128,8 @@ imap_readline (f_imap_t f_imap)
   while (f_imap->nl == NULL);
 
   /* Conversion \r\n --> \n\0  */
-  if (f_imap->nl > f_imap->buffer)
+  /* FIXME: This should be done transparently by the TCP stream */
+  if (f_imap->nl > f_imap->buffer && f_imap->nl[-1] == '\r')
     {
       *(f_imap->nl - 1) = '\n';
       *(f_imap->nl) = '\0';
@@ -2100,9 +2281,7 @@ imap_parse (f_imap_t f_imap)
 			 initial capabilities list.  This makes it unnecessary
 			 for a client to send a separate CAPABILITY command if
 			 it recognizes this response.  */
-		      if (f_imap->capa)
-			free (f_imap->capa);
-		      f_imap->capa = strdup (cruft);
+		      parse_capa (f_imap, cruft);
 		    }
 		  else if (strcasecmp (subtag, "NEWNAME") == 0)
 		    {
@@ -2222,15 +2401,13 @@ imap_parse (f_imap_t f_imap)
 	    }
 	  else if (strcasecmp (response, "CAPABILITY") == 0)
 	    {
-	      if (f_imap->capa)
-		free (f_imap->capa);
-	      f_imap->capa = strdup (remainder);
+	      parse_capa (f_imap, remainder);
 	    }
 	  else if (strcasecmp (remainder, "EXISTS") == 0)
 	    {
 	      f_imap->selected->messages_count = strtol (response, NULL, 10);
 	    }
-	  else if (strcasecmp (remainder, "EXPUNGE") == 0)
+	  else if (strcasecmp (remainder, "EXPUNGED") == 0)
 	    {
 	      unsigned int msgno = strtol (response, NULL, 10);
 	      status = imap_expunge (f_imap, msgno);
@@ -2293,9 +2470,11 @@ imap_parse (f_imap_t f_imap)
 		  observable_t observable = NULL;
 		  folder_get_observable (f_imap->folder, &observable);
 		  observable_notify (observable, MU_EVT_AUTHORITY_FAILED);
+		  status = MU_ERR_AUTH_FAILURE;
 		}
+	      else
+		status = EINVAL;
 	      mu_error ("NO/Bad Tagged: %s %s\n", response, remainder);
-	      status = EINVAL;
 	    }
 	}
       f_imap->ptr = f_imap->buffer;
