@@ -75,7 +75,7 @@ static int  imap_copy_message     __P ((mailbox_t, message_t));
 static int  imap_submessage_size  __P ((msg_imap_t, size_t *));
 static int  imap_message_size     __P ((message_t, size_t *));
 static int  imap_message_lines    __P ((message_t, size_t *));
-static int  imap_message_fd       __P ((stream_t, int *));
+static int  imap_message_fd       __P ((stream_t, int *, int *));
 static int  imap_message_read     __P ((stream_t , char *, size_t, off_t, size_t *));
 static int  imap_message_uid      __P ((message_t, size_t *));
 
@@ -103,10 +103,11 @@ static int  imap_body_read        __P ((stream_t, char *, size_t, off_t,
 					size_t *));
 static int  imap_body_size        __P ((body_t, size_t *));
 static int  imap_body_lines       __P ((body_t, size_t *));
-static int  imap_body_fd          __P ((stream_t, int *));
+static int  imap_body_fd          __P ((stream_t, int *, int *));
 
 /* Helpers.  */
-static int  imap_get_fd           __P ((msg_imap_t, int *));
+static int  imap_get_fd2          __P ((msg_imap_t msg_imap, int *pfd1,
+					int *pfd2));
 static int  imap_get_message0     __P ((msg_imap_t, message_t *));
 static int  fetch_operation       __P ((f_imap_t, msg_imap_t, char *, size_t, size_t *));
 static void free_subparts         __P ((msg_imap_t));
@@ -830,9 +831,6 @@ imap_is_updated (mailbox_t mailbox)
 }
 
 
-/* It is only here that the Deleted flags are sent.  Expunge is not
-   call rather the mailbox is close explicitely, letting the server
-   do the expunge without sending the notifications.  It's faster.  */
 static int
 imap_expunge (mailbox_t mailbox)
 {
@@ -879,11 +877,11 @@ imap_expunge (mailbox_t mailbox)
       MAILBOX_DEBUG0 (m_imap->mailbox, MU_DEBUG_PROT, f_imap->buffer);
       f_imap->state = IMAP_NO_STATE;
 
-      /* We are not sending EXPUNGE, rather we close the mailbox
-	 which will purge.  */
-    case IMAP_CLOSE:
-    case IMAP_CLOSE_ACK:
-      status = mailbox_imap_close (mailbox);
+    case IMAP_EXPUNGE:
+    case IMAP_EXPUNGE_ACK:
+      status = imap_writeline (f_imap, "g%d EXPUNGE\r\n", f_imap->seq++);
+      CHECK_ERROR (f_imap, status);
+      status = imap_send (f_imap);
       CHECK_EAGAIN (f_imap, status);
 
       /* Rescan after expunging but do not trigger the observers.  */
@@ -893,7 +891,7 @@ imap_expunge (mailbox_t mailbox)
       CHECK_EAGAIN (f_imap, status);
 
     default:
-      /* mu_error ("imap_expunge: unknow state\n"); */
+      /* mu_error ("imap_expunge: unknown state\n"); */
       break;
     }
 
@@ -1331,11 +1329,11 @@ imap_message_uid (message_t msg, size_t *puid)
 }
 
 static int
-imap_message_fd (stream_t stream, int * pfd)
+imap_message_fd (stream_t stream, int *pfd, int *pfd2)
 {
   message_t msg = stream_get_owner (stream);
   msg_imap_t msg_imap = message_get_owner (msg);
-  return imap_get_fd (msg_imap, pfd);
+  return imap_get_fd2 (msg_imap, pfd, pfd2);
 }
 
 /* Mime.  */
@@ -2003,23 +2001,24 @@ imap_body_read (stream_t stream, char *buffer, size_t buflen, off_t offset,
 }
 
 static int
-imap_body_fd (stream_t stream, int *pfd)
+imap_body_fd (stream_t stream, int *pfd, int *pfd2)
 {
   body_t body = stream_get_owner (stream);
   message_t msg = body_get_owner (body);
   msg_imap_t msg_imap = message_get_owner (msg);
-  return imap_get_fd (msg_imap, pfd);
+  return imap_get_fd2 (msg_imap, pfd, pfd2);
 }
 
 
 static int
-imap_get_fd (msg_imap_t msg_imap, int *pfd)
+imap_get_fd2 (msg_imap_t msg_imap, int *pfd1, int *pfd2)
 {
   if (   msg_imap
       && msg_imap->m_imap
       && msg_imap->m_imap->f_imap
       && msg_imap->m_imap->f_imap->folder)
-    return stream_get_fd (msg_imap->m_imap->f_imap->folder->stream, pfd);
+    return stream_get_fd2 (msg_imap->m_imap->f_imap->folder->stream,
+			   pfd1, pfd2);
   return EINVAL;
 }
 
