@@ -61,6 +61,7 @@ extern char *strchrnul __P((const char *s, int c_in));
 #define MH_USER_PROFILE ".mh_profile"
 #define MH_GLOBAL_PROFILE "mh-profile"
 #define MH_CONTEXT_FILE "context"
+#define DEFAULT_ALIAS_FILE MHLIBDIR "/MailAliases"
 
 #define is_true(arg) ((arg)==NULL||(arg)[0] == 'y')
 
@@ -123,6 +124,8 @@ enum mh_opcode
      Format: mhop_fmtspec number */
   mhop_fmtspec,
 
+  /* 18. Noop */
+  mhop_nop
 };    
 
 enum mh_type
@@ -239,14 +242,19 @@ char *mh_current_folder __P((void));
 char *mh_global_sequences_get __P((const char *name, const char *defval));
 int mh_global_sequences_set __P((const char *name, const char *value));
 void mh_global_save_state __P((void));
+int mh_global_profile_iterate __P((mh_context_iterator fp, void *data));
 int mh_global_context_iterate __P((mh_context_iterator fp, void *data));
 int mh_global_sequences_iterate __P((mh_context_iterator fp, void *data));
+void mh_global_sequences_drop __P((void));
 
+int mh_interactive_mode_p __P((void));
 int mh_getyn __P((const char *fmt, ...));
+int mh_getyn_interactive __P((const char *fmt, ...));
 int mh_check_folder __P((char *pathname, int confirm));
 
 int mh_format __P((mh_format_t *fmt, message_t msg, size_t msgno,
 		   size_t width, char **pret));
+int mh_format_str __P((mh_format_t *fmt, char *str, size_t width, char **pret));
 void mh_format_dump __P((mh_format_t *fmt));
 int mh_format_parse __P((char *format_str, mh_format_t *fmt));
 void mh_format_debug __P((int val));
@@ -280,6 +288,7 @@ void mh_msgset_reverse __P((mh_msgset_t *msgset));
 void mh_msgset_negate __P((mailbox_t mbox, mh_msgset_t *msgset));
 int mh_msgset_current __P((mailbox_t mbox, mh_msgset_t *msgset, int index));
 void mh_msgset_free __P((mh_msgset_t *msgset));
+void mh_msgset_uids __P((mailbox_t mbox, mh_msgset_t *msgset));
 
 char *mh_get_dir __P((void));
 char *mh_expand_name __P((const char *base, const char *name, int is_folder));
@@ -305,6 +314,19 @@ int mh_file_copy __P((const char *from, const char *to));
 char *mh_draft_name __P((void));
 char *mh_create_message_id __P((int));
 int mh_whom __P((char *filename, int check));
+void mh_set_reply_regex __P((const char *str));
+
+int mh_alias_read __P((char *name, int fail));
+int mh_alias_get __P((char *name, list_t *return_list));
+int mh_alias_get_address __P((char *name, address_t *addr, int *incl));
+int mh_alias_get_alias __P((char *uname, list_t *return_list));
+int mh_read_aliases __P((void));
+int mh_alias_expand __P((char *str, address_t *paddr, int *incl));
+
+typedef int (*mh_alias_enumerator_t) __PMT((char *alias, list_t names,
+					    void *data));
+void mh_alias_enumerate __P((mh_alias_enumerator_t fun, void *data));
+
 
 void mh_annotate __P((message_t msg, char *field, char *text, int date));
 
@@ -322,9 +344,8 @@ void mh_seq_add __P((char *name, mh_msgset_t *mset, int flags));
 int mh_seq_delete __P((char *name, mh_msgset_t *mset, int flags));
 char *mh_seq_read __P((char *name, int flags));
 
-int mhdraft_stream_create __P((stream_t *stream, stream_t src, int flags));
-
 void mh_comp_draft __P((char *formfile, char *defformfile, char *draftfile));
 int check_draft_disposition __P((struct mh_whatnow_env *wh, int use_draft));
 
+void ali_parse_error __P((char *fmt, ...));
 
