@@ -152,9 +152,9 @@ static const short fmt_yyrhs[] = {    17,
 static const short fmt_yyrline[] = { 0,
     70,    76,    77,    80,   105,   106,   112,   117,   124,   125,
    126,   132,   147,   153,   159,   159,   159,   187,   188,   195,
-   196,   211,   215,   216,   220,   252,   256,   259,   265,   279,
-   286,   293,   304,   305,   308,   313,   321,   327,   336,   340,
-   343
+   196,   211,   215,   216,   220,   252,   256,   259,   265,   277,
+   284,   291,   302,   303,   306,   311,   319,   325,   334,   338,
+   341
 };
 #endif
 
@@ -978,32 +978,30 @@ case 29:
 #line 266 "mh_fmtgram.y"
 {
 	      /* False branch of an if-block */
-	      fmt_yyval.elif_list.cond = mh_code_number (0);
-	      mh_code_op (mhop_num_asgn);
+	      fmt_yyval.elif_list.cond = mh_code_op (mhop_num_asgn);
 	      /* Jump over the true branch */
 	      mh_code_op (mhop_branch);
-	      mh_code_op (3);
+	      mh_code_op (2);
 	      /* True branch */
-	      fmt_yyval.elif_list.end = mh_code_number (1);
-	      mh_code_op (mhop_num_asgn);
+	      fmt_yyval.elif_list.end = mh_code_op (mhop_num_asgn);
 	    ;
     break;}
 case 30:
-#line 280 "mh_fmtgram.y"
+#line 278 "mh_fmtgram.y"
 {
 	      in_escape++;
 	      fmt_yyval.pc = pc;
 	    ;
     break;}
 case 31:
-#line 287 "mh_fmtgram.y"
+#line 285 "mh_fmtgram.y"
 {
 	      mh_code_op (mhop_branch);
 	      fmt_yyval.pc = mh_code_op (0);
 	    ;
     break;}
 case 32:
-#line 294 "mh_fmtgram.y"
+#line 292 "mh_fmtgram.y"
 {
 	      in_escape--;
 	      if (fmt_yyvsp[0].type == mhtype_str)
@@ -1014,14 +1012,14 @@ case 32:
 	    ;
     break;}
 case 35:
-#line 309 "mh_fmtgram.y"
+#line 307 "mh_fmtgram.y"
 {
 	      fmt_yyval.elif_list.cond = 0;
 	      fmt_yyval.elif_list.end = 0;
 	    ;
     break;}
 case 36:
-#line 314 "mh_fmtgram.y"
+#line 312 "mh_fmtgram.y"
 {
 	      fmt_yyval.elif_list.cond = fmt_yyvsp[-1].elif_list.cond;
 	      MHI_NUM(format.prog[fmt_yyvsp[0].pc]) = fmt_yyvsp[-1].elif_list.end;
@@ -1029,7 +1027,7 @@ case 36:
 	    ;
     break;}
 case 37:
-#line 322 "mh_fmtgram.y"
+#line 320 "mh_fmtgram.y"
 {
 	      fmt_yyval.elif_list.cond = fmt_yyvsp[-2].pc;
 	      MHI_NUM(format.prog[fmt_yyvsp[-1].pc]) = pc - fmt_yyvsp[-1].pc + 2;
@@ -1037,7 +1035,7 @@ case 37:
 	    ;
     break;}
 case 38:
-#line 328 "mh_fmtgram.y"
+#line 326 "mh_fmtgram.y"
 {
 	      MHI_NUM(format.prog[fmt_yyvsp[-1].pc]) = pc - fmt_yyvsp[-1].pc + 2;
 	      fmt_yyval.elif_list.cond = fmt_yyvsp[-4].elif_list.cond;
@@ -1046,13 +1044,13 @@ case 38:
 	    ;
     break;}
 case 39:
-#line 337 "mh_fmtgram.y"
+#line 335 "mh_fmtgram.y"
 {
 	      fmt_yyval.pc = 0;
 	    ;
     break;}
 case 41:
-#line 344 "mh_fmtgram.y"
+#line 342 "mh_fmtgram.y"
 {
 	      fmt_yyval.pc = pc;
 	    ;
@@ -1279,7 +1277,7 @@ fmt_yyerrhandle:
     }
   return 1;
 }
-#line 349 "mh_fmtgram.y"
+#line 347 "mh_fmtgram.y"
 
 
 static char *start;
@@ -1554,6 +1552,8 @@ mh_code_builtin (mh_builtin_t *bp, int argtype)
 		  break;
 		  
 		case mhtype_str:
+		  if (bp->optarg == MHA_OPT_CLEAR)
+		    mh_code_string ("");
 		  /* mhtype_none means that the argument was an escape,
 		     which has left its string value (if any) in the
 		     arg_str register. Therefore, there's no need to
