@@ -18,6 +18,7 @@
 /* This module implements execution of MH format strings. */
 
 #include <mh.h>
+#include <mailutils/mime.h>
 
 #ifdef HAVE_STRINGS_H
 # include <strings.h>
@@ -261,7 +262,12 @@ print_simple_segment (struct mh_machine *mach, size_t width,
 
   rest = width - mach->ind;
   if (rest == 0)
-    return;
+    {
+      if (len == 1 && str[0] == '\n')
+	put_string (mach, str, len);
+      return;
+    }
+  
   if (len > rest)
     len = rest;
 
@@ -442,7 +448,7 @@ mh_format (mh_format_t *fmt, message_t msg, size_t msgno,
   mach.message = msg;
   mach.msgno = msgno;
   
-  mach.width = width;
+  mach.width = width - 1; /* Count the newline */
   mach.pc = 1;
   obstack_init (&mach.stk);
   list_create (&mach.addrlist);
@@ -454,6 +460,9 @@ mh_format (mh_format_t *fmt, message_t msg, size_t msgno,
       mh_opcode_t opcode;
       switch (opcode = MHI_OPCODE(mach.prog[mach.pc++]))
 	{
+	case mhop_nop:
+	  break;
+	  
 	case mhop_stop:
 	  mach.stop = 1;
 	  break;
@@ -505,7 +514,7 @@ mh_format (mh_format_t *fmt, message_t msg, size_t msgno,
 	    header_t hdr = NULL;
 	    char *value = NULL;
 	    message_get_header (mach.message, &hdr);
-	    header_aget_value (hdr, strobj_ptr (&mach.arg_str), &value);
+	    header_aget_value_unfold (hdr, strobj_ptr (&mach.arg_str), &value);
 	    strobj_free (&mach.arg_str);
 	    if (value)
 	      {
@@ -513,7 +522,10 @@ mh_format (mh_format_t *fmt, message_t msg, size_t msgno,
 		mach.arg_str.size = len + 1;
       		compress_ws (value, &len);
 		mach.arg_str.ptr = value;
+		mach.arg_num = 1;
 	      }
+	    else
+	      mach.arg_num = 0;
 	  }
 	  break;
 
@@ -636,6 +648,10 @@ mh_format_dump (mh_format_t *fmt)
       printf ("% 4.4ld: ", (long) pc);
       switch (opcode = MHI_OPCODE(prog[pc++]))
 	{
+	case mhop_nop:
+	  printf ("nop");
+	  break;
+	  
 	case mhop_stop:
 	  printf ("stop");
 	  stop = 1;
@@ -871,8 +887,8 @@ static void
 builtin_me (struct mh_machine *mach)
 {
   char *s = mh_my_email ();
-  strobj_free (&mach->arg_str);
-  strobj_create (&mach->arg_str, s);
+  strobj_realloc (&mach->arg_str, strlen (s) + 3);
+  sprintf (strobj_ptr (&mach->arg_str), "<%s>", s);
 }
 
 static void
@@ -1708,7 +1724,7 @@ builtin_formataddr (struct mh_machine *mach)
   address_t addr, dest;
   size_t size;
   
-  if (strobj_is_null (&mach->reg_str))
+  if (strobj_len (&mach->reg_str) == 0)
     dest = NULL;
   else if (address_create (&dest, strobj_ptr (&mach->reg_str)))
     return;
@@ -1751,10 +1767,12 @@ builtin_formataddr (struct mh_machine *mach)
 	}
     }
   list_append (mach->addrlist, addr);
-  address_to_string (dest, NULL, 0, &size);
-  strobj_realloc (&mach->reg_str, size + 1);
-  address_to_string (dest, strobj_ptr (&mach->reg_str), size + 1, NULL);
-  address_destroy (&dest);
+  if (address_to_string (dest, NULL, 0, &size) == 0)
+    {
+      strobj_realloc (&mach->reg_str, size + 1);
+      address_to_string (dest, strobj_ptr (&mach->reg_str), size + 1, NULL);
+      address_destroy (&dest);
+    }
 }
 
 /*      putaddr    literal        print str address list with
@@ -1777,37 +1795,10 @@ builtin_putaddr (struct mh_machine *mach)
 static void
 builtin_unre (struct mh_machine *mach)
 {
-  char *start, *p;
-
-  for (p = strobj_ptr (&mach->arg_str); *p; p++)
-    if (!isspace (*p))
-      break;
-
-  start = p;
-  
-  if (strncasecmp (p, "re", 2) == 0)
+  char *p;
+  int rc = munre_subject (strobj_ptr (&mach->arg_str), &p);
+  if (rc == 0 && p != strobj_ptr (&mach->arg_str))
     {
-      if (p[2] == ':')
-	p += 3;
-      else if (p[2] == '[')
-	{
-	  for (p += 3; *p; p++)
-	    if (*p == ']' || !isdigit (*p))
-	      break;
-	  if (*p == ']' && p[1] == ':') 
-	    p += 2;
-	  else
-	    p = start;
-	}
-      else
-	p = start;
-    }
-
-  if (p != strobj_ptr (&mach->arg_str))
-    {
-      for (; *p && isspace (*p); p++)
-	;
-      
       p = strdup (p);
       strobj_free (&mach->arg_str);
       strobj_create (&mach->arg_str, p);
@@ -1815,6 +1806,83 @@ builtin_unre (struct mh_machine *mach)
     }
 }  
 
+static void
+builtin_isreply (struct mh_machine *mach)
+{
+  int rc;
+  
+  if (strobj_is_null (&mach->arg_str))
+    {
+      header_t hdr = NULL;
+      char *value = NULL;
+      message_get_header (mach->message, &hdr);
+      
+      header_aget_value (hdr, MU_HEADER_SUBJECT, &value);
+      rc = munre_subject (value, NULL);
+      free (value);
+    }
+  else
+    rc = munre_subject (strobj_ptr (&mach->arg_str), NULL);
+
+  mach->arg_num = !rc;
+}
+
+static void
+decode_string (strobj_t *obj)
+{
+  char *charset = NULL;
+  char *tmp;
+  int rc;
+
+  if (strobj_is_null (obj))
+    return;
+  charset = mh_global_profile_get ("Charset", NULL);
+  if (!charset)
+    return;
+  if (strcasecmp (charset, "auto") == 0)
+    {
+      /* Try to deduce the charset from LC_ALL variable */
+
+      tmp = getenv ("LC_ALL");
+      if (tmp)
+	{
+	  char *sp;
+	  char *lang;
+	  char *terr;
+
+	  lang = strtok_r (tmp, "_", &sp);
+	  terr = strtok_r (NULL, ".", &sp);
+	  charset = strtok_r (NULL, "@", &sp);
+
+	  if (!charset)
+	    charset = mu_charset_lookup (lang, terr);
+	}
+    }
+
+  if (!charset)
+    return;
+  
+  rc = rfc2047_decode (charset, strobj_ptr (obj), &tmp);
+  if (!rc)
+    {
+      strobj_free (obj);
+      strobj_create (obj, tmp);
+      free (tmp);
+    }
+}
+
+static void
+builtin_decode (struct mh_machine *mach)
+{
+  decode_string (&mach->arg_str);
+}
+
+static void
+builtin_reply_regex (struct mh_machine *mach)
+{
+  mh_set_reply_regex (strobj_ptr (&mach->arg_str));
+}
+
 int
 mh_decode_rcpt_flag (const char *arg)
 {
@@ -1846,6 +1914,9 @@ static void
 builtin_concat (struct mh_machine *mach)
 {
   size_t size = strobj_len (&mach->arg_str);
+
+  if (size == 0)
+    return;
   compress_ws (strobj_ptr (&mach->arg_str), &size);
   if (strobj_len (&mach->reg_str) == 0)
     strobj_copy (&mach->reg_str, &mach->arg_str);
@@ -1864,10 +1935,27 @@ builtin_concat (struct mh_machine *mach)
 static void
 builtin_printhdr (struct mh_machine *mach)
 {
+  char *tmp = NULL;
+  size_t s = 0;
+  
   if (!strobj_is_null (&mach->arg_str))
-    print_hdr_string (mach, strobj_ptr (&mach->arg_str));
+    {
+      s = strobj_len (&mach->arg_str);
+      tmp = strdup (strobj_ptr (&mach->arg_str));
+    }
+  
   if (!strobj_is_null (&mach->reg_str))
-    print_hdr_string (mach, strobj_ptr (&mach->reg_str));
+    {
+      s += strobj_len (&mach->reg_str) + 1;
+      tmp = realloc (tmp, s);
+      strcat (tmp, strobj_ptr (&mach->reg_str));
+    }
+
+  if (tmp)
+    {
+      print_hdr_string (mach, tmp);
+      free (tmp);
+    }
 }
 
 static void
@@ -1876,9 +1964,11 @@ builtin_in_reply_to (struct mh_machine *mach)
   char *value;
 
   strobj_free (&mach->arg_str);
-  mu_rfc2822_in_reply_to (mach->message, &value);
-  strobj_create (&mach->arg_str, value);
-  free (value);
+  if (mu_rfc2822_in_reply_to (mach->message, &value) == 0)
+    {
+      strobj_create (&mach->arg_str, value);
+      free (value);
+    }
 }
 
 static void
@@ -1887,9 +1977,11 @@ builtin_references (struct mh_machine *mach)
   char *value;
 
   strobj_free (&mach->arg_str);
-  mu_rfc2822_references (mach->message, &value);
-  strobj_create (&mach->arg_str, value);
-  free (value);
+  if (mu_rfc2822_references (mach->message, &value) == 0)
+    {
+      strobj_create (&mach->arg_str, value);
+      free (value);
+    }
 }
 
 static void
@@ -1935,20 +2027,20 @@ mh_builtin_t builtin_tab[] = {
   { "divide",   builtin_divide,   mhtype_num,  mhtype_num },
   { "modulo",   builtin_modulo,   mhtype_num,  mhtype_num },
   { "num",      builtin_num,      mhtype_num,  mhtype_num },
-  { "lit",      builtin_lit,      mhtype_str,  mhtype_str, 1 },
+  { "lit",      builtin_lit,      mhtype_str,  mhtype_str,  MHA_OPT_CLEAR },
   { "getenv",   builtin_getenv,   mhtype_str,  mhtype_str },
   { "profile",  builtin_profile,  mhtype_str,  mhtype_str },
-  { "nonzero",  builtin_nonzero,  mhtype_num,  mhtype_num, 1 },
-  { "zero",     builtin_zero,     mhtype_num,  mhtype_num, 1 },
-  { "null",     builtin_null,     mhtype_num,  mhtype_str, 1 },
-  { "nonnull",  builtin_nonnull,  mhtype_num,  mhtype_str, 1 },
-  { "comp",     builtin_comp,     mhtype_num,  mhtype_str },
-  { "compval",  builtin_compval,  mhtype_num,  mhtype_str },
-  { "trim",     builtin_trim,     mhtype_str,  mhtype_str, 1 },
-  { "putstr",   builtin_putstr,   mhtype_none,  mhtype_str, 1 },
-  { "putstrf",  builtin_putstrf,  mhtype_none,  mhtype_str, 1 },
-  { "putnum",   builtin_putnum,   mhtype_none,  mhtype_num, 1 },
-  { "putnumf",  builtin_putnumf,  mhtype_none,  mhtype_num, 1 },
+  { "nonzero",  builtin_nonzero,  mhtype_num,  mhtype_num,  MHA_OPTARG },
+  { "zero",     builtin_zero,     mhtype_num,  mhtype_num,  MHA_OPTARG },
+  { "null",     builtin_null,     mhtype_num,  mhtype_str,  MHA_OPTARG },
+  { "nonnull",  builtin_nonnull,  mhtype_num,  mhtype_str,  MHA_OPTARG },
+  { "comp",     builtin_comp,     mhtype_num,  mhtype_str,  MHA_OPTARG },
+  { "compval",  builtin_compval,  mhtype_num,  mhtype_str },	   
+  { "trim",     builtin_trim,     mhtype_str,  mhtype_str,  MHA_OPTARG },
+  { "putstr",   builtin_putstr,   mhtype_none,  mhtype_str, MHA_OPTARG },
+  { "putstrf",  builtin_putstrf,  mhtype_none,  mhtype_str, MHA_OPTARG },
+  { "putnum",   builtin_putnum,   mhtype_none,  mhtype_num, MHA_OPTARG },
+  { "putnumf",  builtin_putnumf,  mhtype_none,  mhtype_num, MHA_OPTARG },
   { "sec",      builtin_sec,      mhtype_num,  mhtype_str },
   { "min",      builtin_min,      mhtype_num,  mhtype_str },
   { "hour",     builtin_hour,     mhtype_num,  mhtype_str },
@@ -1986,17 +2078,20 @@ mh_builtin_t builtin_tab[] = {
   { "path",     builtin_path,     mhtype_str,  mhtype_str },
   { "ingrp",    builtin_ingrp,    mhtype_num,  mhtype_str },
   { "gname",    builtin_gname,    mhtype_str,  mhtype_str},
-  { "formataddr", builtin_formataddr, mhtype_none, mhtype_str, 1 },
+  { "formataddr", builtin_formataddr, mhtype_none, mhtype_str, MHA_OPTARG },
   { "putaddr",  builtin_putaddr,  mhtype_none, mhtype_str },
   { "unre",     builtin_unre,     mhtype_str,  mhtype_str },
   { "rcpt",     builtin_rcpt,     mhtype_num,  mhtype_str },
-  { "concat",   builtin_concat,   mhtype_none, mhtype_str,     1 },
+  { "concat",   builtin_concat,   mhtype_none, mhtype_str,     MHA_OPTARG },
   { "printhdr", builtin_printhdr, mhtype_none, mhtype_str },
   { "in_reply_to", builtin_in_reply_to, mhtype_str,  mhtype_none },
   { "references", builtin_references, mhtype_str,  mhtype_none },
   { "package",  builtin_package,  mhtype_str, mhtype_none },
   { "package_string",  builtin_package_string,  mhtype_str, mhtype_none },
   { "version",  builtin_version,  mhtype_str, mhtype_none },
+  { "reply_regex", builtin_reply_regex, mhtype_none, mhtype_str },
+  { "isreply", builtin_isreply, mhtype_num, mhtype_str, MHA_OPTARG },
+  { "decode", builtin_decode, mhtype_str, mhtype_str },
   { 0 }
 };
 
