@@ -318,7 +318,7 @@ _mh_msgset_parse (mailbox_t mbox, mh_msgset_t *msgset, int argc, char **argv)
   msglist = calloc (msgcnt, sizeof(*msglist));
   for (i = 0, msgno = 0; i < argc; i++)
     {
-      char *p = NULL;
+      char *p = NULL, *q;
       size_t start, end;
       size_t msg_first, n;
       long num;
@@ -380,9 +380,15 @@ _mh_msgset_parse (mailbox_t mbox, mh_msgset_t *msgset, int argc, char **argv)
 	      break;
 	      
 	    case ':':
-	      num = strtoul (p+1, &p, 0);
-	      if (*p)
+	      num = strtoul (p+1, &q, 0);
+	      if (*q)
 		msgset_abort (argv[i]);
+	      if (p[1] != '+' && p[1] != '-')
+		{
+		  if (strncmp (argv[i], "last:", 5) == 0
+		      || strncmp (argv[i], "prev:", 5) == 0)
+		    num = -num;
+		}
 	      end = start + num;
 	      if (end < start)
 		{
@@ -587,7 +593,6 @@ mh_msgset_negate (mailbox_t mbox, mh_msgset_t *msgset)
   list = calloc (total, sizeof (list[0]));
   if (!list)
     mh_err_memory (1);
-
   for (i = 1, msgno = 0; i <= total; i++)
     {
       if (!mh_msgset_member (msgset, i))
@@ -605,3 +610,14 @@ mh_msgset_negate (mailbox_t mbox, mh_msgset_t *msgset)
   msgset->list = list;
 }
 
+void
+mh_msgset_uids (mailbox_t mbox, mh_msgset_t *msgset)
+{
+  size_t i;
+  for (i = 0; i < msgset->count; i++)
+    {
+      message_t msg;
+      mailbox_get_message (mbox, msgset->list[i], &msg);
+      mh_message_number (msg, &msgset->list[i]);
+    }
+}
