@@ -21,7 +21,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 
-const char *argp_program_version = "mhl (" PACKAGE_STRING ")";
+const char *program_version = "mhl (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH mhl\v"
 "Use -help to obtain the list of traditional MH options.");
 static char args_doc[] = N_("[files]");
@@ -46,6 +46,8 @@ static struct argp_option options[] = {
    N_("Use given PROG instead of the default") },
   {"nomoreproc", ARG_NOMOREPROC, NULL, 0,
    N_("Disable use of moreproc program") },
+  {"license", ARG_LICENSE, 0,      0,
+   N_("Display software license"), -1},
   { NULL }
 };
    
@@ -62,8 +64,8 @@ struct mh_option mh_option[] = {
 };
 
 static int interactive;  /* Using interactive output */
-static int bell = 1;     /* Ring the bell after each page of output */
-static int clear = 0;    /* Clear the screen after each page of output */
+static int mhl_fmt_flags; /* MHL format flags. Controlled by --bell 
+                             and --clear */
 static int length = 40;  /* Length of output page */
 static int width = 80;   /* Width of output page */
 static char *formfile = MHLIBDIR "/mhl.format";
@@ -77,25 +79,26 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
 {
   switch (key)
     {
-    case '+':
     case ARG_FOLDER:
       current_folder = arg;
       break;
 
     case ARG_BELL:
-      bell = is_true (arg);
+      if (is_true (arg))
+        mhl_fmt_flags |= MHL_BELL;
       break;
       
     case ARG_NOBELL:
-      bell = 0;
+      mhl_fmt_flags &= ~MHL_BELL;
       break;
       
     case ARG_CLEAR:
-      clear = is_true (arg);
+      if (is_true (arg))
+        mhl_fmt_flags |= MHL_CLEARSCREEN;
       break;
       
     case ARG_NOCLEAR:
-      clear = 0;
+      mhl_fmt_flags &= ~MHL_CLEARSCREEN;
       break;
       
     case ARG_FORM:
@@ -128,6 +131,10 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       nomoreproc = 1;
       break;
       
+    case ARG_LICENSE:
+      mh_license (argp_program_version);
+      break;
+
     default:
       return 1;
     }
@@ -198,7 +205,7 @@ list_message (char *name, stream_t output)
     }
   else
     {
-      mhl_format_run (format, width, length, clear, bell, msg, output);
+      mhl_format_run (format, width, length, mhl_fmt_flags, msg, output);
       message_unref (msg);
     }
 }
@@ -212,6 +219,7 @@ main (int argc, char **argv)
   interactive = isatty (1) && isatty (0);
   
   mu_init_nls ();
+  mu_argp_init (program_version, NULL);
   mh_argp_parse (argc, argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
@@ -226,7 +234,7 @@ main (int argc, char **argv)
     nomoreproc = 1;
 
   if (!interactive)
-    bell = 0;
+    mhl_fmt_flags &= ~MHL_BELL;
   
   output = open_output ();
   
