@@ -24,7 +24,7 @@
 #include <errno.h>
 #include <fcntl.h>
 
-const char *argp_program_version = "refile (" PACKAGE_STRING ")";
+const char *program_version = "refile (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH refile\v"
 "Options marked with `*' are not yet implemented.\n"
 "Use -help to obtain the list of traditional MH options.");
@@ -36,14 +36,17 @@ static struct argp_option options[] = {
    N_("Specify folder to operate upon")},
   {"draft",   ARG_DRAFT, NULL, 0,
    N_("Use <mh-dir>/draft as the source message")},
-  {"link",    ARG_LINK, N_("BOOL"), OPTION_ARG_OPTIONAL,
-   N_("* Preserve the source folder copy")},
+  {"copy",    ARG_LINK, N_("BOOL"), OPTION_ARG_OPTIONAL,
+   N_("Preserve the source folder copy.")},
+  {"link",    0, NULL, OPTION_ALIAS, NULL},
   {"preserve", ARG_PRESERVE, N_("BOOL"), OPTION_ARG_OPTIONAL,
    N_("* Try to preserve message sequence numbers")},
   {"source", ARG_SOURCE, N_("FOLDER"), 0,
    N_("Specify source folder. FOLDER will become the current folder after the program exits.")},
   {"src", 0, NULL, OPTION_ALIAS, NULL},
   {"file", ARG_FILE, N_("FILE"), 0, N_("Use FILE as the source message")},
+  {"license", ARG_LICENSE, 0,      0,
+   N_("Display software license"), -1},
   { 0 }
 };
 
@@ -57,7 +60,7 @@ struct mh_option mh_option[] = {
   { 0 }
 };
 
-int link_flag = 1;
+int link_flag = 0;
 int preserve_flag = 0;
 char *source_file = NULL;
 list_t folder_name_list = NULL;
@@ -149,13 +152,12 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
 {
   switch (key)
     {
-    case '+':
     case ARG_FOLDER: 
       add_folder (arg);
       break;
 
     case ARG_DRAFT:
-      source_file = mh_expand_name (NULL, "draft", 0);
+      source_file = "draft";
       break;
 
     case ARG_LINK:
@@ -174,6 +176,10 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       source_file = arg;
       break;
       
+    case ARG_LICENSE:
+      mh_license (argp_program_version);
+      break;
+
     default:
       return 1;
     }
@@ -218,26 +224,41 @@ main (int argc, char **argv)
   int index;
   mh_msgset_t msgset;
   mailbox_t mbox;
-  int status;
+  int status, i, j;
 
   /* Native Language Support */
   mu_init_nls ();
 
+  mu_argp_init (program_version, NULL);
   mh_argp_parse (argc, argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
+  argc -= index;
+  argv += index;
+
+  /* Collect any surplus folders */
+  for (i = j = 0; i < argc; i++)
+    {
+      if (argv[i][0] == '+')
+	add_folder (argv[i]);
+      else
+	argv[j++] = argv[i];
+    }
+  argv[j] = NULL;
+  argc = j;
+  
   open_folders ();
 
   if (source_file)
     {
       message_t msg;
       
-      if (index < argc)
+      if (argc > 0)
 	{
 	  mh_error (_("both message set and source file given"));
 	  exit (1);
 	}
-      msg = mh_file_to_message (NULL, source_file);
+      msg = mh_file_to_message (mu_path_folder_dir, source_file);
       refile (msg);
       if (!link_flag)
 	unlink (source_file);
@@ -246,13 +267,13 @@ main (int argc, char **argv)
   else
     {
       mbox = mh_open_folder (current_folder, 0);
-      mh_msgset_parse (mbox, &msgset, argc - index, argv + index, "cur");
+      mh_msgset_parse (mbox, &msgset, argc, argv, "cur");
 
       status = mh_iterate (mbox, &msgset, refile_iterator, NULL);
  
-  mailbox_expunge (mbox);
-  mailbox_close (mbox);
-  mailbox_destroy (&mbox);
+      mailbox_expunge (mbox);
+      mailbox_close (mbox);
+      mailbox_destroy (&mbox);
     }
 
   close_folders ();
